const webpack = require('webpack');
const chalk = require('chalk');
const appConfig = require('../common/appConfig');
const { runCallback } = require('../common/build');
const getEnvironmentInfo = require('../common/extendEnv');
const extendWebpackConfigFromProject = require('../common/extendWebpackConfigFromProject');
const apiConfigFactory = require('./webpack.prod.api');

process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';

process.on('unhandledRejection', err => {
  // eslint-disable-next-line @typescript-eslint/no-throw-literal
  throw err;
});

const extendEnv = getEnvironmentInfo();

function build() {
  const apiConfig = extendWebpackConfigFromProject(apiConfigFactory, appConfig, extendEnv, {
    isServer: true,
    isHotServer: false,
    isProd: true
  });
  // eslint-disable-next-line no-console
  console.log(chalk.green('Creating an optimized production build...'));
  const apiCompiler = webpack(apiConfig);
  return new Promise((resolve, reject) => {
    apiCompiler.run(runCallback(resolve, reject));
  });
}

build()
  .then(
    buildInfo => {
      if (buildInfo.warnings.length) {
        // eslint-disable-next-line no-console
        console.log(chalk.yellow('Compiled with warnings.\n'));
        // eslint-disable-next-line no-console
        console.log(buildInfo.warnings.join('\n\n'));
        // eslint-disable-next-line no-console
        console.log(`\nSearch for the ${chalk.underline(chalk.yellow('keywords'))} to learn more about each warning.`);
        // eslint-disable-next-line no-console
        console.log(`To ignore, add ${chalk.cyan('// eslint-disable-next-line')} to the line before.\n`);
      } else {
        // eslint-disable-next-line no-console
        console.log(chalk.green('Compiled successfully.\n'));
      }
    },
    err => {
      // eslint-disable-next-line no-console
      console.log(chalk.red('Failed to compile.\n'));
      // eslint-disable-next-line no-console
      console.log(err);
      process.exit(1);
    }
  )
  .catch(err => {
    // eslint-disable-next-line no-console
    console.log(chalk.red('Failed to compile.\n'));
    // eslint-disable-next-line no-console
    console.log(err);
    process.exit(1);
  });
