const nodemon = require('nodemon');
const open = require('open');
const getEnvironmentInfo = require('../common/extendEnv');
const appConfig = require('../common/appConfig');
const { getProjectPath, parseArgsToJson } = require('../common/utils');

const extendEnv = getEnvironmentInfo();

const PORT = process.env.PORT || parseArgsToJson.port || appConfig.startDefaultPort;
const startOpenPath = appConfig.startOpenPath ? appConfig.startOpenPath : '';

process.env.BABEL_ENV = 'development';
process.env.NODE_ENV = 'development';

const options = {
  // 配置文件
  script: getProjectPath(appConfig.apiEntry),
  restartable: 're',
  verbose: true,
  delay: 1000,
  watch: appConfig.hotApiWatch,
  env: {
    PORT: PORT,
    BABEL_ENV: 'development',
    NODE_ENV: 'development',
    ...extendEnv.raw
  }
};

let isOpened = false;

nodemon(options)
  .on('start', () => {
    // 启动时的事件
    // eslint-disable-next-line no-console
    console.log('Server app has started');
    if (startOpenPath && !isOpened) {
      setTimeout(() => {
        open(`http://localhost:${PORT}${startOpenPath}`);
      }, 3000);
      isOpened = true;
    }
  })
  .on('quit', () => {
    // 退出时的事件
    // eslint-disable-next-line no-console
    console.log('Server app has quit');
    process.exit();
  })
  .on('restart', () => {
    // 重启时的事件
    // eslint-disable-next-line no-console
    console.log('Server app has restarted');
  });
