const { merge } = require('webpack-merge');
const { getProjectPath, deleteFolders } = require('../common/utils');
const webpackCommon = require('../common/webpack.common');

module.exports = function (appConfig, extendEnv, { isServer, isHotServer, isProd }) {
  const result = merge(webpackCommon(appConfig, extendEnv, { isProd, isServer, isHotServer }), {
    target: 'node',
    entry: getProjectPath(appConfig.apiEntry),
    output: {
      path: getProjectPath(appConfig.apiOutputPath),
      publicPath: '/',
      filename: 'index.js',
      libraryTarget: 'commonjs2',
      chunkFilename: '[name].[chunkhash].js'
    },
    module: {
      rules: [
        {
          oneOf: [
            {
              test: /\.(scss|css|sass)$/,
              use: [
                {
                  loader: require.resolve('ignore-loader')
                }
              ]
            }
          ]
        }
      ]
    }
  });

  const buildServerOutput = getProjectPath(result.output.path);
  deleteFolders(buildServerOutput);

  return result;
};
