const { merge } = require('webpack-merge');
const { getProjectPath, deleteFolders } = require('../common/utils');
const baseConfig = require('./webpack.base');

module.exports = function (appConfig, extendEnv, { isServer, isHotServer, isProd }) {
  const result = merge(
    baseConfig(appConfig, extendEnv, {
      isServer,
      isHotServer,
      isProd
    }),
    {
      target: 'node',
      entry: getProjectPath(appConfig.serverEntryAtSsr),
      output: {
        path: getProjectPath(appConfig.serverOutputPathAtSsr),
        publicPath: '/',
        filename: 'index.js',
        libraryTarget: 'commonjs2',
        chunkFilename: `${appConfig.baseStaticJsPath}/[name].[chunkhash].js`
      },
      module: {
        rules: [
          {
            oneOf: [
              {
                test: /\.(scss|css|sass)$/,
                use: [
                  {
                    loader: require.resolve('ignore-loader')
                  }
                ]
              }
            ]
          }
        ]
      }
    }
  );

  const buildServerOutput = getProjectPath(result.output.path);
  deleteFolders(buildServerOutput);

  return result;
};
